﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Automation;
using System.Windows;
using System.Runtime.InteropServices;


namespace UITestClient
{    
    public class UIWrapper
    {
        private string _pathFromRoot;
        private static int retryCount = 40;
        private static int retryDelay = 250;
       
        [DllImport("user32.dll")]
        internal static extern bool IsWindowEnabled(IntPtr hWnd);


        public UIWrapper(string path)
        {
            this._pathFromRoot = path;
        }

        private AutomationElement ObtainElementFromPath()
        {
            var tags = _pathFromRoot.Split('\\');
            AutomationElement parent = AutomationElement.RootElement;
            for (int i = 0; i < tags.Length; i++)
            {
                parent= ObtainChildElement(parent, tags[i]);
                if (parent == null)
                {
                    return null;
                }
            }
            return parent;
        }

        private AutomationElement ObtainChildElement(AutomationElement parent, string tag)
        {
            int tagSeperatorIndex = tag.IndexOf("::");
            string typeTag = tag.Substring(0, tagSeperatorIndex);
            string strTag = tag.Substring(tagSeperatorIndex + "::".Length);

            TreeWalker walker = new TreeWalker(Automation.ControlViewCondition);
            var start = walker.GetFirstChild(parent);
            var end = walker.GetLastChild(parent);
            if (start != null && end != null)
            {
                string info;
                bool _useID = false;
                if (strTag.StartsWith("["))
                {
                    _useID = true;
                    strTag = strTag.Substring(1);
                    strTag = strTag.Substring(0, strTag.Length - 1);
                }

                while (start!=null)
                {
                    if (_useID)
                    {
                        info = start.GetCurrentPropertyValue(AutomationElement.AutomationIdProperty).ToString();
                    }
                    else
                    {
                        info = start.GetCurrentPropertyValue(AutomationElement.NameProperty).ToString();
                    }
                    if (string.Compare(info, strTag, true) == 0)
                    {
                        ControlType type = (ControlType)start.GetCurrentPropertyValue(AutomationElement.ControlTypeProperty);
                        if (string.Compare(type.LocalizedControlType, typeTag,true)==0)
                        {
                            return start;
                        }
                    }

                    start = walker.GetNextSibling(start);
                }
            }

            return null;
        }

        private AutomationElement ObtainElement()
        {
            for (int i = 0; i < retryCount; i++)
            {
                var ret = ObtainElementFromPath();
                if (ret == null)
                {
                    System.Threading.Thread.Sleep(retryDelay);
                }
                else
                {
                    return ret;
                }
            }

            throw new TimeoutException(string.Format("Cannot obtain element with path {0}", this._pathFromRoot));
        }

        private AutomationElement GetTopWindow(AutomationElement item)
        {
            TreeWalker walker = new TreeWalker(Condition.TrueCondition);
            while (true)
            {
                var parent = walker.GetParent(item);
                if (parent.Equals(AutomationElement.RootElement))
                {
                    return item;
                }
                item = parent;
            }
        }

        private void EnsureTopWindowOnTop()
        {
            var item = ObtainElement();
            var top = GetTopWindow(item);

            WindowPattern wndptn = (WindowPattern)top.GetCurrentPattern(WindowPattern.Pattern);
            if (wndptn == null)
            {
                return;
            }

            var currentStates = wndptn.Current.WindowVisualState;
            if (currentStates != WindowVisualState.Minimized)
            {
                wndptn.SetWindowVisualState(currentStates);
            }
            else
            {
                wndptn.SetWindowVisualState(WindowVisualState.Normal);
            }

            return;
        }

        private void SimpleWaitForReady()
        {
            var item = ObtainElement();
            int handle = (int)item.GetCurrentPropertyValue(AutomationElement.NativeWindowHandleProperty);
            if (handle == 0)
            {                
                var top = GetTopWindow(item);
                handle = (int)top.GetCurrentPropertyValue(AutomationElement.NativeWindowHandleProperty);
            }

            if (handle == 0)
            {
                return;
            }

            IntPtr hwnd = new IntPtr(handle);

            for (int i = 0; i < retryCount; i++)
            {
                if (IsWindowEnabled(hwnd))
                {
                    return;
                }
                System.Threading.Thread.Sleep(retryDelay);
            }
        }

        public void Click()
        {
            EnsureTopWindowOnTop();
            SimpleWaitForReady();
            var point = Clicker.GetClickPoint(ObtainElement());
            Clicker.Click(point);
        }

        public void Invoke()
        {
            SimpleWaitForReady();
            var ele=ObtainElement();
            InvokePattern pattern = (InvokePattern)ele.GetCurrentPattern(InvokePattern.Pattern);
            if (pattern == null)
            {
                throw new InvalidOperationException("This element does not support InvokePattern");
            }
            pattern.Invoke();
        }

        public string Name
        {
            get
            {
                SimpleWaitForReady();
                return ObtainElement().GetCurrentPropertyValue(AutomationElement.NameProperty).ToString();
            }
        }

        public string Value
        {
            get
            {
                SimpleWaitForReady();
                ValuePattern pattern = (ValuePattern)ObtainElement().GetCurrentPattern(ValuePattern.Pattern);
                if (pattern == null)
                {
                    throw new InvalidOperationException("This element does not support ValuePattern");
                }
                return pattern.Current.Value;
            }
        }

    }   
}
